@@ -1,25 +1,34 @@
 ##' Wilcoxon Rank Sum and Signed Rank Test for Clustered Data
 ##'
-##' Performs one-sample and two-sample Wilcoxon test for clutered data
+##' Performs one-sample and two-sample Wilcoxon test for clustered data
 ##' on vectors of data.
 ##'
 ##' @param x A numeric vector of data values or a formula. Non-finite (e.g.,
 ##'     infinite or missing) values will be omitted.
 ##' @param y An optional numeric vector of data values, non-finite
-##'     values will be omitted. 
-##' @param cluster An optional numeric vector of cluster id. 
-##' @param group An optional numeric vector of treatment id. 
-##' @param stratum An opptional numeric vector of stratum id. Only
+##'     values will be omitted.
+##' @param cluster An optional numeric vector of cluster id.
+##' @param group An optional numeric vector of treatment id.
+##' @param stratum An optional numeric vector of stratum id. Only
 ##' available for \code{rgl} rank sum test when treatment is assigned at
 ##'     cluster level.
 ##' @param method A character string specifying the method of
-##'     clustered wilcoxon rank test to be use, should be one of
+##'     clustered Wilcoxon rank test to be use, should be one of
 ##'     \code{"rgl"} or \code{"ds"}.
 ##' @param paired A logical indicating whether you want a paired test.
-##' @param exact A logical indicating whether an exact p-value should
-##'     be computed. Only available for \code{rgl} signed rank test
-##'     and \code{rgl} rank sum test when treatment is assigned at
-##'     cluster leve.
+##' @param exact A logical indicating if a permutation test is to be
+##'     used. If it is set as \code{FALSE}, then test based on
+##'     large-sample theory is carried out. If it is set as
+##'     \code{TRUE}, then a permutation test is carried out. There are
+##'     two available permutation tests: exact permutation test is available for
+##'     \code{rgl} signed rank test and \code{rgl} rank sum test when
+##'     treatment is assigned at cluster level, random permutation
+##'     test is available for all tests. For more details look at the
+##'     description of the argument \code{B}.
+##' @param B An integer. If is set as 0, then exact permutation test is
+##'     carried out. If is set as a positive integer, then random permutation
+##'     is carried out, and B is the number of replicates drawn for
+##'     the permutation test. Default is set as 2000 to ran a random permutation test.
 ##' @param formula A formula of the form \code{lhs ~ rhs} where the
 ##'     \code{lhs} is the measurements and
 ##'     the \code{rhs} is of the form group + \code{cluster}(x1) +
@@ -27,7 +36,7 @@
 ##'     \code{x1} and \code{x2} are cluster id and stratum id in the
 ##'     data frame \code{data}. For clustered signed rank test, the
 ##'     \code{rhs} only contains \code{cluster}{x1}.
-##' @param data An optional dataframe containing the variables.
+##' @param data An optional data frame containing the variables.
 ##' @param subset An optional vector specifying a subset of
 ##'     observations to be used.
 ##' @param na.action A function which indicates what should happen
@@ -42,14 +51,14 @@
 ##' @details The formula interface is to both clustered signed rank
 ##'     test and clustered rank sum test.
 ##'
-##' 
+##'
 ##' The default of \code{cluster} id is that there is one member in
 ##'     each cluster. Both balanced data (identical cluster size) and
 ##'     unbalanced data (different cluster sizes) are
 ##'     supported in all tests provided in this package. For clustered
 ##'     rank sum test, the data can either have treatment assigned at
 ##'     cluster level or individual level.
-##' 
+##'
 ##' If both \code{x} and \code{y} are given or only \code{x} is given
 ##'     and \code{paired} is \code{TRUE}, a clustered Wilcoxon signed
 ##'     rank test of the null that the distribution of \code{x - y}
@@ -72,7 +81,7 @@
 ##'
 ##' The exact test is only available for RGL signed rank test and  RGL
 ##'     rank sum test when treatment is assigned at cluster level.
-##' 
+##'
 ##' @return A list with class \code{"htest"} containing the following components, for different test the components may vary:
 ##'
 ##' \item{Rstat}{the value of the rank statistic with a name describing it.}
@@ -85,17 +94,17 @@
 ##' \item{method}{the type of test applied.}
 ##' \item{data.name}{a character string giving the names of the data.}
 ##' \item{balance}{a logical indicating whether the data set is balanced.}
-##' \item{n.group}{number of treatment, will be returned if there are more than 2 treatment groups and \code{ds} method is used.}
+##' \item{ngroup}{number of treatment, will be returned if there are more than 2 treatment groups and \code{ds} method is used.}
 ##' \item{df}{degrees of freedom of chi-square distribution, will be returned when there are more than 2 treatment groups and \code{ds} method is used.}
-##' \item{n}{number of observations}
-##' \item{cn}{number of clusters}
+##' \item{nobs}{number of observations}
+##' \item{nclus}{number of clusters}
 
 #' @section Warning:
 #' This function can use large amounts of memory and stack if 'exact =
 #'     TRUE' and one sample is large (and even crash R if the stack
-#'     limit is exceeded). Not recommended for data set 
+#'     limit is exceeded). Not recommended for data set
 #'     with number of clusters more than 50.
-#' 
+#'
 #' @examples
 #' ## Clustered signed rank test using RGL method.
 #' data(crsd)
@@ -118,14 +127,14 @@
 #' Bernard Rosner, Robert J. Glynn, Mei-Ling T. Lee (2003)
 #' \emph{Incorporation of Clustering Effects for the Wilcoxon Rank
 #' Sum Test: A Large-Sample Approach}. Biometrics, \bold{59}, 1089-1098.
-#' 
+#'
 #' Bernard Rosner, Robert J. Glynn, Mei-Ling T. Lee (2006)
-#' \emph{Extension of the Rank Sum Test for Clustered Data: 
+#' \emph{Extension of the Rank Sum Test for Clustered Data:
 #' Two-Group Comparisons with Group}. Biometrics, \bold{62}, 1251-1259.
-#' 
+#'
 #' Somnath Datta, Glen A. Satten (2005) \emph{Rank-Sum Tests for Clustered Data}.
 #' Journal of the American Statistical Association, \bold{100}, 908-915.
-#' 
+#'
 #' Somath Datta, Glen A. Satten (2008) \emph{A Signed-Rank test for Clustered Data}.
 #' Biometric, \bold{64}, 501-507.
 #'
@@ -133,7 +142,7 @@
 #' @importFrom stats lm ecdf pnorm qnorm var  pchisq setNames lag
 #' @importFrom MASS ginv
 #' @importFrom Rcpp evalCpp
-#' @useDynLib clusrank 
+#' @useDynLib clusrank
 #' @export
 
 clusWilcox.test <- function(x, ...) {
@@ -165,12 +174,13 @@ clusWilcox.test.formula <- function(formula, data = parent.frame(), subset = NUL
                                     na.action = na.omit,
                                     alternative = c("two.sided", "less", "greater"),
                                     mu = 0, paired = FALSE, exact = FALSE,
-                                    method = c("rgl", "ds"),  ...) {
+                                    B = 2000, method = c("rgl", "ds"),
+                                    ...) {
     if (missing(formula) || (length(formula) != 3L)) {
         stop("'formula' missing or incorrect")
     }
     m <- match.call(expand.dots = FALSE)
-    
+
 
     if (is.matrix(eval(m$data, parent.frame()))) {
         m$data <- as.data.frame(data)
@@ -191,19 +201,19 @@ clusWilcox.test.formula <- function(formula, data = parent.frame(), subset = NUL
         if ("na.action" %in% names(m)) na.ind <- which(names(m) == "na.action")
         else na.ind <- 5
     }
-    
-    
+
+
     m <- if (missing(data)) m <- m[c(1 : 2, subset.ind, na.ind)]
          else m <- m[c(1 : 3, subset.ind, na.ind)]
     mf <- eval(m, parent.frame())
-    Terms <- terms(mf) ## Delete 
+    Terms <- terms(mf) ## Delete
 
     x.name <- rownames(attr(m$formula, "factors"))[1]
     DNAME <- paste0(x.name, ";")
     response <- attr(terms(mf), "response")
     x <- mf[[response]]
     n.obs <- length(x)
-    
+
     group <- extractTerm("group", mf, n.obs, paired)
     DNAME <- paste0(DNAME, group[["name"]])
     group <- group[["var"]]
@@ -215,19 +225,20 @@ clusWilcox.test.formula <- function(formula, data = parent.frame(), subset = NUL
     stratum <- extractTerm("stratum", mf, n.obs, paired)
     DNAME <- paste0(DNAME, stratum[["name"]])
     stratum <- stratum[["var"]]
-    
-    
+
+
     ## please use tab key to keep the indentation correct
     if (!missing(data)) {
         DNAME <- paste(DNAME, "(from", paste0(m$data, ")"))
     }
-    
+
     y <- do.call("clusWilcox.test.default",
                  c( list(x = x, cluster = cluster,
                          group = group, stratum = stratum,
                          DNAME = DNAME, paired = paired,
                          alternative = alternative,
-                         method = method, mu = mu, exact = exact),
+                         method = method, mu = mu, exact = exact,
+                         B = B),
                    list(...)))
     return(y)
 }
@@ -240,12 +251,13 @@ clusWilcox.test.formula <- function(formula, data = parent.frame(), subset = NUL
 clusWilcox.test.default <- function(x, y = NULL, cluster = NULL,
                                     group = NULL, stratum = NULL, data = NULL,
                                     alternative = c("two.sided", "less", "greater"),
-                                    mu = 0, paired = FALSE, exact = FALSE,
+                                    mu = 0, paired = FALSE, exact = FALSE, B
+                                    = 1,
                                     method = c("rgl", "ds"), ...) {
     alternative <- match.arg(alternative)
     method <- match.arg(method)
     pars <- as.list(match.call()[-1])
-    
+
     if (!missing(mu) && ((length(mu) > 1L) || !is.finite(mu)))
         stop("'mu' must be a single number")
     DNAME <- list(...)$"DNAME"
@@ -272,9 +284,9 @@ clusWilcox.test.default <- function(x, y = NULL, cluster = NULL,
             DNAME <- paste0(DNAME, extractName("y", pars))
             DNAME <- paste0(DNAME, extractName("group", pars))
             DNAME <- paste0(DNAME, extractName("cluster", pars))
-            DNAME <- paste0(DNAME, extractName("stratum", pars))    
+            DNAME <- paste0(DNAME, extractName("stratum", pars))
         }
-    } 
+    }
 
 
     if (!is.null(y)) {
@@ -295,7 +307,7 @@ clusWilcox.test.default <- function(x, y = NULL, cluster = NULL,
     if (is.null(group) && paired == FALSE) {
         stop("'group' is required for the clustered rank sum test")
     }
-    
+
     if (!is.null(group) && paired == TRUE) {
         warning("'group' will be ignored for the clustered signed rank test")
     }
@@ -315,11 +327,11 @@ clusWilcox.test.default <- function(x, y = NULL, cluster = NULL,
     cluster <- cluster[OK]
     group <- group[OK]
     stratum <- stratum[OK]
-    
+
     if (length(x) < 1L) {
         stop("not enough (finite) 'x' observation")
     }
-    
+
     if (paired == TRUE) {
         if (length(table(stratum)) > 1L) {
             warning("'stratum' will be ignored for the clustered signed rank test")
@@ -329,58 +341,63 @@ clusWilcox.test.default <- function(x, y = NULL, cluster = NULL,
         if (method == "rgl") {
             METHOD <- paste(METHOD, "using Rosner-Glynn-Lee method", sep = " ")
             arglist <- setNames(list(x, cluster, alternative, mu, METHOD,
-                                     DNAME, exact),
+                                     DNAME, exact, B),
                                 c("x", "cluster", "alternative",
                                   "mu",
-                                  "METHOD", "DNAME",  "exact"))
+                                  "METHOD", "DNAME",  "exact", "B"))
             result <- do.call("clusWilcox.test.signedrank.rgl", c(arglist))
             return(result)
         }
-        
+
         if (method == "ds") {
+            if (exact == TRUE & B == 0) {
+                warning("Exact exactutation test is not available for 'ds' method. Will run the large-sample test.")
+                exact <- FALSE
+            }
             METHOD <- paste(METHOD, "using Datta-Satten method", sep = " ")
-            arglist <- setNames(list(x, cluster, alternative, mu, METHOD, DNAME),
+            arglist <- setNames(list(x, cluster, alternative, mu, exact, B, METHOD, DNAME),
                                 c("x", "cluster", "alternative",
-                                  "mu", "METHOD", "DNAME"))
+                                  "mu", "exact", "B", "METHOD", "DNAME"))
+
             result <-  do.call("clusWilcox.test.signedrank.ds",
                                c(arglist))
             return(result)
         } else {
             stop("Method should be one of 'rgl' and 'ds'")
         }
-        
+
     } else {
         METHOD <- "Clustered Wilcoxon rank sum test"
         if ((method) == "rgl") {
             METHOD <- paste( METHOD, "using Rosner-Glynn-Lee method", sep = " ")
             arglist <- setNames(list(x, cluster, group, stratum,
                                      alternative, mu, DNAME,
-                                     METHOD, exact),
+                                     METHOD, exact, B),
                                 c("x", "cluster", "group", "stratum",
                                   "alternative", "mu", "DNAME", "METHOD",
-                                  "exact"))
+                                  "exact", "B"))
             result <- do.call("clusWilcox.test.ranksum.rgl", c(arglist))
             return(result)
         }
-        
+
         if ((method) == "ds") {
             METHOD <- paste(METHOD, "using Datta-Satten method", sep = " ")
-            ## FIXME: The length of the list and the name do not match!
             arglist <- setNames(list(x, cluster, group, mu, alternative,
-                                     DNAME, METHOD),
+                                     DNAME, METHOD, exact, B),
                                 c("x", "cluster", "group", "mu",
-                                  "alternative", "DNAME", "METHOD"))
-            if (exact == TRUE) {
-                warning(" No exact test is provided for 'ds' method")
+                                  "alternative", "DNAME", "METHOD", "exact", "B"))
+            if (exact == TRUE & B == 0) {
+                warning("Exact exactutation test is not available for 'ds' method. Will run the large-sample test.")
+                exact <- FALSE
             }
-            
+
             if (length(table(stratum)) > 1L) {
                 warning("'stratum' will be ignored for the clustered rank sum test, 'ds' method")
             }
             result <- do.call("clusWilcox.test.ranksum.ds", c(arglist))
             return(result)
         }
-        
+
         else {
             stop("Method should be one of 'rgl' and 'ds'")
         }
